@@ -164,7 +164,7 @@ jcat_context_get_keyring_path (JcatContext *self)
  * @error: #GError, or %NULL
  *
  * Gets the engine for a specific engine kind, setting up the context
- * automartically if requried.
+ * automatically if required.
  *
  * Returns: (transfer full): #JcatEngine, or %NULL for unavailable
  *
@@ -270,13 +270,16 @@ jcat_context_verify_item (JcatContext *self,
 	/* all checksum engines must verify */
 	for (guint i = 0; i < blobs->len; i++) {
 		JcatBlob *blob = g_ptr_array_index (blobs, i);
+		g_autoptr(GError) error_local = NULL;
 		g_autoptr(JcatEngine) engine = NULL;
 		g_autoptr(JcatResult) result = NULL;
 
 		/* get engine */
-		engine = jcat_context_get_engine (self, jcat_blob_get_kind (blob), error);
-		if (engine == NULL)
-			return NULL;
+		engine = jcat_context_get_engine (self, jcat_blob_get_kind (blob), &error_local);
+		if (engine == NULL) {
+			g_debug ("%s", error_local->message);
+			continue;
+		}
 		if (jcat_engine_get_method (engine) != JCAT_BLOB_METHOD_CHECKSUM)
 			continue;
 		result = jcat_engine_self_verify (engine, data, jcat_blob_get_data (blob), flags, error);
@@ -302,9 +305,11 @@ jcat_context_verify_item (JcatContext *self,
 		g_autoptr(JcatEngine) engine = NULL;
 		g_autoptr(JcatResult) result = NULL;
 
-		engine = jcat_context_get_engine (self, jcat_blob_get_kind (blob), error);
-		if (engine == NULL)
-			return NULL;
+		engine = jcat_context_get_engine (self, jcat_blob_get_kind (blob), &error_local);
+		if (engine == NULL) {
+			g_debug ("%s", error_local->message);
+			continue;
+		}
 		if (jcat_engine_get_method (engine) != JCAT_BLOB_METHOD_SIGNATURE)
 			continue;
 		result = jcat_engine_pubkey_verify (engine, data, jcat_blob_get_data (blob), flags, &error_local);
