@@ -61,10 +61,14 @@ jcat_item_func (void)
 	g_autoptr(JcatItem) item = NULL;
 	const gchar *str_perfect =
 		"JcatItem:\n"
-		"  ID:                    filename.bin\n";
+		"  ID:                    filename.bin\n"
+		"  AliasId:               foo.bin\n";
 
 	/* sanity check */
 	item = jcat_item_new ("filename.bin");
+	jcat_item_add_alias_id (item, "foo.bin");
+	jcat_item_add_alias_id (item, "bar.bin");
+	jcat_item_remove_alias_id (item, "bar.bin");
 	g_assert_cmpstr (jcat_item_get_id (item), ==, "filename.bin");
 
 	/* to string */
@@ -103,6 +107,9 @@ jcat_file_func (void)
 		"  \"Items\" : [\n"
 		"    {\n"
 		"      \"Id\" : \"firmware.bin\",\n"
+		"      \"AliasIds\" : [\n"
+		"        \"foo.bin\"\n"
+		"      ],\n"
 		"      \"Blobs\" : [\n"
 		"        {\n"
 		"          \"Kind\" : 2,\n"
@@ -142,6 +149,7 @@ jcat_file_func (void)
 	jcat_item_add_blob (item, blob1);
 	jcat_item_add_blob (item, blob2);
 	jcat_item_add_blob (item, blob2);
+	jcat_item_add_alias_id (item, "foo.bin");
 	blobs1 = jcat_item_get_blobs (item);
 	g_assert_cmpint (blobs1->len, ==, 2);
 	blobs2 = jcat_item_get_blobs_by_kind (item, JCAT_BLOB_KIND_GPG);
@@ -213,10 +221,10 @@ jcat_sha1_engine_func (void)
 	g_assert_no_error (error);
 	g_assert_nonnull (engine);
 	g_assert_cmpint (jcat_engine_get_kind (engine), ==, JCAT_BLOB_KIND_SHA1);
-	g_assert_cmpint (jcat_engine_get_verify_kind (engine), ==, JCAT_ENGINE_VERIFY_KIND_CHECKSUM);
+	g_assert_cmpint (jcat_engine_get_method (engine), ==, JCAT_BLOB_METHOD_CHECKSUM);
 
 	/* verify checksum */
-	fn_pass = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin", NULL);
+	fn_pass = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin", NULL);
 	data_fwbin = jcat_get_contents_bytes (fn_pass, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fwbin);
@@ -230,7 +238,7 @@ jcat_sha1_engine_func (void)
 	g_assert_cmpstr (jcat_result_get_authority (result_pass), ==, NULL);
 
 	/* verify will fail */
-	fn_fail = g_build_filename (TESTDATADIR_SRC, "meson.build",NULL);
+	fn_fail = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin.asc", NULL);
 	data_fail = jcat_get_contents_bytes (fn_fail, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fail);
@@ -271,10 +279,10 @@ jcat_sha256_engine_func (void)
 	g_assert_no_error (error);
 	g_assert_nonnull (engine);
 	g_assert_cmpint (jcat_engine_get_kind (engine), ==, JCAT_BLOB_KIND_SHA256);
-	g_assert_cmpint (jcat_engine_get_verify_kind (engine), ==, JCAT_ENGINE_VERIFY_KIND_CHECKSUM);
+	g_assert_cmpint (jcat_engine_get_method (engine), ==, JCAT_BLOB_METHOD_CHECKSUM);
 
 	/* verify checksum */
-	fn_pass = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin", NULL);
+	fn_pass = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin", NULL);
 	data_fwbin = jcat_get_contents_bytes (fn_pass, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fwbin);
@@ -288,7 +296,7 @@ jcat_sha256_engine_func (void)
 	g_assert_cmpstr (jcat_result_get_authority (result_pass), ==, NULL);
 
 	/* verify will fail */
-	fn_fail = g_build_filename (TESTDATADIR_SRC, "meson.build",NULL);
+	fn_fail = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin.asc", NULL);
 	data_fail = jcat_get_contents_bytes (fn_fail, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fail);
@@ -340,7 +348,7 @@ jcat_gpg_engine_func (void)
 
 	/* set up context */
 	jcat_context_set_keyring_path (context, "/tmp/libjcat-self-test/var");
-	pki_dir = g_build_filename (TESTDATADIR_SRC, "pki", NULL);
+	pki_dir = g_test_build_filename (G_TEST_DIST, "pki", NULL);
 	jcat_context_add_public_keys (context, pki_dir);
 
 	/* get engine */
@@ -348,7 +356,7 @@ jcat_gpg_engine_func (void)
 	g_assert_no_error (error);
 	g_assert_nonnull (engine);
 	g_assert_cmpint (jcat_engine_get_kind (engine), ==, JCAT_BLOB_KIND_GPG);
-	g_assert_cmpint (jcat_engine_get_verify_kind (engine), ==, JCAT_ENGINE_VERIFY_KIND_SIGNATURE);
+	g_assert_cmpint (jcat_engine_get_method (engine), ==, JCAT_BLOB_METHOD_SIGNATURE);
 
 	/* to string */
 	str = jcat_engine_to_string (engine);
@@ -356,7 +364,7 @@ jcat_gpg_engine_func (void)
 	g_assert_cmpstr (str, ==, str_perfect);
 
 	/* verify with GnuPG */
-	fn_pass = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin", NULL);
+	fn_pass = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin", NULL);
 	data_fwbin = jcat_get_contents_bytes (fn_pass, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fwbin);
@@ -371,7 +379,7 @@ jcat_gpg_engine_func (void)
 			 "3FC6B804410ED0840D8F2F9748A6D80E4538BAC2");
 
 	/* verify will fail with GnuPG */
-	fn_fail = g_build_filename (TESTDATADIR_SRC, "meson.build",NULL);
+	fn_fail = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin.asc", NULL);
 	data_fail = jcat_get_contents_bytes (fn_fail, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fail);
@@ -385,6 +393,60 @@ jcat_gpg_engine_func (void)
 #endif
 }
 
+static void
+jcat_gpg_engine_msg_func (void)
+{
+#ifdef ENABLE_GPG
+	g_autofree gchar *fn = NULL;
+	g_autofree gchar *pki_dir = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autoptr(GBytes) data_sig = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(JcatContext) context = jcat_context_new ();
+	g_autoptr(JcatEngine) engine = NULL;
+	g_autoptr(JcatResult) result = NULL;
+	const gchar *sig =
+	"-----BEGIN PGP MESSAGE-----\n"
+	"owGbwMvMwMEovmZX76/pfOKMp0WSGOLOX3/ikZqTk6+jUJ5flJOiyNXJaMzCwMjB\n"
+	"ICumyCJmt5VRUil28/1+z1cwbaxMID0MXJwCMJG4RxwMLUYXDkUad34I3vrT8+X2\n"
+	"m+ZyHyMWnTiQYaQb/eLJGqbiAJc5Jr4a/PPqHNi7auwzGsKsljebabjtnJRzpDr0\n"
+	"YvwrnmmWLJUnTzjM3MH5Kn+RzqXkywsYdk9yD2OUdLy736CiemFMdcuF02lOZvPU\n"
+	"HaTKl76wW62QH8Lr8yGMQ1Xgc6nC2ZwUhvctky7NOZtc1T477uBTL81p31ZmaIUJ\n"
+	"paS8uWZl8UzX5sFsqQi37G1TbDc8Cm+oU/yRkFj2pLBzw367ncsa4n7EqEWu1yrN\n"
+	"yD39LUeErePdqfKCG+xhL6WkWt5ZJ/6//XnjouXhl5Z4tWspT49MtNp5d3aDQ43c\n"
+	"mnbresn6A7KMZgdOiwIA\n"
+	"=a9ui\n"
+	"-----END PGP MESSAGE-----\n";
+
+	/* set up context */
+	jcat_context_set_keyring_path (context, "/tmp/libjcat-self-test/var");
+	pki_dir = g_test_build_filename (G_TEST_DIST, "pki", NULL);
+	jcat_context_add_public_keys (context, pki_dir);
+
+	/* get engine */
+	engine = jcat_context_get_engine (context, JCAT_BLOB_KIND_GPG, &error);
+	g_assert_no_error (error);
+	g_assert_nonnull (engine);
+	g_assert_cmpint (jcat_engine_get_kind (engine), ==, JCAT_BLOB_KIND_GPG);
+	g_assert_cmpint (jcat_engine_get_method (engine), ==, JCAT_BLOB_METHOD_SIGNATURE);
+
+	/* verify with GnuPG, which should fail as the signature is not a
+	 * detached signature at all, but gnupg stabs us in the back by returning
+	 * success from gpgme_op_verify() with an empty list of signatures */
+	fn = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin", NULL);
+	data = jcat_get_contents_bytes (fn, &error);
+	g_assert_no_error (error);
+	g_assert_nonnull (data);
+	data_sig = g_bytes_new_static (sig, strlen (sig));
+	result = jcat_engine_pubkey_verify (engine, data, data_sig,
+					    JCAT_VERIFY_FLAG_NONE, &error);
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_FAILED);
+	g_assert_null (result);
+#else
+	g_test_skip ("no GnuPG support enabled");
+#endif
+}
+
 static void
 jcat_pkcs7_engine_func (void)
 {
@@ -406,7 +468,7 @@ jcat_pkcs7_engine_func (void)
 
 	/* set up context */
 	jcat_context_set_keyring_path (context, "/tmp/libjcat-self-test/var");
-	pki_dir = g_build_filename (TESTDATADIR_SRC, "pki", NULL);
+	pki_dir = g_test_build_filename (G_TEST_DIST, "pki", NULL);
 	jcat_context_add_public_keys (context, pki_dir);
 
 	/* get engine */
@@ -414,14 +476,14 @@ jcat_pkcs7_engine_func (void)
 	g_assert_no_error (error);
 	g_assert_nonnull (engine);
 	g_assert_cmpint (jcat_engine_get_kind (engine), ==, JCAT_BLOB_KIND_PKCS7);
-	g_assert_cmpint (jcat_engine_get_verify_kind (engine), ==, JCAT_ENGINE_VERIFY_KIND_SIGNATURE);
+	g_assert_cmpint (jcat_engine_get_method (engine), ==, JCAT_BLOB_METHOD_SIGNATURE);
 
 	/* verify with a signature from the old LVFS */
-	fn_pass = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin", NULL);
+	fn_pass = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin", NULL);
 	data_fwbin = jcat_get_contents_bytes (fn_pass, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fwbin);
-	fn_sig = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin.p7b", NULL);
+	fn_sig = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin.p7b", NULL);
 	data_sig = jcat_get_contents_bytes (fn_sig, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_sig);
@@ -434,7 +496,7 @@ jcat_pkcs7_engine_func (void)
 	g_assert_cmpstr (jcat_result_get_authority (result_pass), == , "O=Linux Vendor Firmware Project,CN=LVFS CA");
 
 	/* verify will fail with a self-signed signature */
-	sig_fn2 = g_build_filename (TESTDATADIR_DST, "colorhug", "firmware.bin.p7c", NULL);
+	sig_fn2 = g_test_build_filename (G_TEST_BUILT, "colorhug", "firmware.bin.p7c", NULL);
 	blob_sig2 = jcat_get_contents_bytes (sig_fn2, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (blob_sig2);
@@ -445,7 +507,7 @@ jcat_pkcs7_engine_func (void)
 	g_clear_error (&error);
 
 	/* verify will fail with valid signature and different data */
-	fn_fail = g_build_filename (TESTDATADIR_SRC, "meson.build",NULL);
+	fn_fail = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin.asc", NULL);
 	data_fail = jcat_get_contents_bytes (fn_fail, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fail);
@@ -463,6 +525,7 @@ static void
 jcat_pkcs7_engine_self_signed_func (void)
 {
 #ifdef ENABLE_PKCS7
+	static const char payload_str[] = "Hello, world!";
 	g_autofree gchar *str = NULL;
 	g_autoptr(JcatBlob) signature = NULL;
 	g_autoptr(JcatContext) context = jcat_context_new ();
@@ -486,8 +549,7 @@ jcat_pkcs7_engine_self_signed_func (void)
 	g_assert_no_error (error);
 	g_assert_nonnull (engine);
 
-	payload = jcat_get_contents_bytes ("/etc/machine-id", &error);
-	g_assert_no_error (error);
+	payload = g_bytes_new_static (payload_str, sizeof (payload_str));
 	g_assert_nonnull (payload);
 	signature = jcat_engine_self_sign (engine, payload, JCAT_SIGN_FLAG_ADD_TIMESTAMP, &error);
 	g_assert_no_error (error);
@@ -517,7 +579,7 @@ jcat_context_verify_blob_func (void)
 #ifdef ENABLE_PKCS7
 	g_autofree gchar *fn_pass = NULL;
 	g_autofree gchar *fn_sig = NULL;
-	g_autofree gchar *pki_dir;
+	g_autofree gchar *pki_dir = NULL;
 	g_autoptr(GBytes) data_fwbin = NULL;
 	g_autoptr(GBytes) data_sig = NULL;
 	g_autoptr(GError) error = NULL;
@@ -533,7 +595,7 @@ jcat_context_verify_blob_func (void)
 
 	/* set up context */
 	jcat_context_set_keyring_path (context, "/tmp");
-	pki_dir = g_build_filename (TESTDATADIR_SRC, "pki", NULL);
+	pki_dir = g_test_build_filename (G_TEST_DIST, "pki", NULL);
 	jcat_context_add_public_keys (context, pki_dir);
 
 	/* get all engines */
@@ -554,11 +616,11 @@ jcat_context_verify_blob_func (void)
 	g_clear_error (&error);
 
 	/* verify blob */
-	fn_pass = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin", NULL);
+	fn_pass = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin", NULL);
 	data_fwbin = jcat_get_contents_bytes (fn_pass, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fwbin);
-	fn_sig = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin.p7b", NULL);
+	fn_sig = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin.p7b", NULL);
 	data_sig = jcat_get_contents_bytes (fn_sig, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_sig);
@@ -582,7 +644,7 @@ jcat_context_verify_item_sign_func (void)
 	JcatResult *result;
 	g_autofree gchar *fn_pass = NULL;
 	g_autofree gchar *fn_sig = NULL;
-	g_autofree gchar *pki_dir;
+	g_autofree gchar *pki_dir = NULL;
 	g_autoptr(GBytes) data_fwbin = NULL;
 	g_autoptr(GBytes) data_sig = NULL;
 	g_autoptr(GError) error = NULL;
@@ -600,7 +662,7 @@ jcat_context_verify_item_sign_func (void)
 
 	/* set up context */
 	jcat_context_set_keyring_path (context, "/tmp");
-	pki_dir = g_build_filename (TESTDATADIR_SRC, "pki", NULL);
+	pki_dir = g_test_build_filename (G_TEST_DIST, "pki", NULL);
 	jcat_context_add_public_keys (context, pki_dir);
 
 	/* get all engines */
@@ -621,11 +683,11 @@ jcat_context_verify_item_sign_func (void)
 	g_clear_error (&error);
 
 	/* verify blob */
-	fn_pass = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin", NULL);
+	fn_pass = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin", NULL);
 	data_fwbin = jcat_get_contents_bytes (fn_pass, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fwbin);
-	fn_sig = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin.p7b", NULL);
+	fn_sig = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin.p7b", NULL);
 	data_sig = jcat_get_contents_bytes (fn_sig, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_sig);
@@ -661,7 +723,7 @@ jcat_context_verify_item_csum_func (void)
 #ifdef ENABLE_PKCS7
 	JcatResult *result;
 	g_autofree gchar *fn_pass = NULL;
-	g_autofree gchar *pki_dir;
+	g_autofree gchar *pki_dir = NULL;
 	g_autoptr(GBytes) data_fwbin = NULL;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(JcatBlob) blob = NULL;
@@ -679,7 +741,7 @@ jcat_context_verify_item_csum_func (void)
 
 	/* set up context */
 	jcat_context_set_keyring_path (context, "/tmp");
-	pki_dir = g_build_filename (TESTDATADIR_SRC, "pki", NULL);
+	pki_dir = g_test_build_filename (G_TEST_DIST, "pki", NULL);
 	jcat_context_add_public_keys (context, pki_dir);
 
 	/* get all engines */
@@ -700,7 +762,7 @@ jcat_context_verify_item_csum_func (void)
 	g_clear_error (&error);
 
 	/* verify blob */
-	fn_pass = g_build_filename (TESTDATADIR_SRC, "colorhug", "firmware.bin", NULL);
+	fn_pass = g_test_build_filename (G_TEST_DIST, "colorhug", "firmware.bin", NULL);
 	data_fwbin = jcat_get_contents_bytes (fn_pass, &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (data_fwbin);
@@ -745,6 +807,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/jcat/engine{sha1}", jcat_sha1_engine_func);
 	g_test_add_func ("/jcat/engine{sha256}", jcat_sha256_engine_func);
 	g_test_add_func ("/jcat/engine{gpg}", jcat_gpg_engine_func);
+	g_test_add_func ("/jcat/engine{gpg-msg}", jcat_gpg_engine_msg_func);
 	g_test_add_func ("/jcat/engine{pkcs7}", jcat_pkcs7_engine_func);
 	g_test_add_func ("/jcat/engine{pkcs7-self-signed}", jcat_pkcs7_engine_self_signed_func);
 	g_test_add_func ("/jcat/context{verify-blob}", jcat_context_verify_blob_func);
